@@ -5,13 +5,15 @@ pub mod release {
         pub allow_dirty: bool,
         pub ignore_instability: bool,
         pub skip_publish: bool,
-        pub no_dry_run_cargo_publish: bool,
+        pub dry_run_cargo_publish: bool,
         /// Pass --no-verify unconditionally to cargo publish. Really just for fixing things
         pub no_verify: bool,
         pub skip_tag: bool,
         pub allow_auto_publish_of_stable_crates: bool,
         pub update_crates_index: bool,
         pub no_bump_on_demand: bool,
+        pub verbose: bool,
+        pub skip_push: bool,
     }
 }
 #[path = "release/mod.rs"]
