@@ -51,7 +51,7 @@ impl Context {
 /// In order to try dealing with https://github.com/sunng87/cargo-release/issues/224 and also to make workspace
 /// releases more selective.
 pub fn release(options: Options, crates: Vec<String>, bump: String, bump_dependencies: String) -> anyhow::Result<()> {
-    if options.no_dry_run_cargo_publish && !options.dry_run {
+    if options.dry_run_cargo_publish && !options.dry_run {
         bail!("The --no-dry-run-cargo-publish flag is only effective without --execute")
     }
     let mut ctx = Context::new(crates, bump, bump_dependencies)?;
@@ -98,7 +98,8 @@ fn release_depth_first(ctx: Context, options: Options) -> anyhow::Result<()> {
         let publishee = package_by_name(meta, publishee_name)?;
 
         let (new_version, commit_id) = perform_single_release(meta, publishee, options, &ctx)?;
-        git::create_version_tag(publishee, &new_version, commit_id, &ctx.repo, options)?;
+        let tag_name = git::create_version_tag(publishee, &new_version, commit_id, &ctx.repo, options)?;
+        git::push_tags_and_head(tag_name, options)?;
     }
 
     if !crates_to_publish_together.is_empty() {
@@ -119,7 +120,7 @@ fn perforrm_multi_version_release(
         .map(|name| {
             let p = package_by_name(meta, &name)?;
             bump_to_valid_version(
-                &p,
+                p,
                 select_publishee_bump_spec(&p.name, ctx),
                 ctx,
                 options.no_bump_on_demand,
@@ -139,18 +140,23 @@ fn perforrm_multi_version_release(
         &crates_to_publish_together,
         bump_spec_may_cause_empty_commits(&ctx.bump) && bump_spec_may_cause_empty_commits(&ctx.bump_dependencies),
         options,
-        &ctx,
+        ctx,
     )?;
 
     crates_to_publish_together.reverse();
+    let mut tag_names = Vec::new();
     while let Some((publishee, new_version)) = crates_to_publish_together.pop() {
         let unpublished_crates: Vec<_> = crates_to_publish_together
             .iter()
             .map(|(p, _)| p.name.to_owned())
             .collect();
+
         cargo::publish_crate(publishee, &unpublished_crates, options)?;
-        git::create_version_tag(publishee, &new_version, commit_id, &ctx.repo, options)?;
+        if let Some(tag_name) = git::create_version_tag(publishee, &new_version, commit_id, &ctx.repo, options)? {
+            tag_names.push(tag_name);
+        };
     }
+    git::push_tags_and_head(tag_names, options)?;
     Ok(())
 }
 
@@ -207,6 +213,7 @@ fn traverse_dependencies_and_find_crates_for_publishing(
     crate_names: &[String],
     ctx: &Context,
     Options {
+        verbose,
         allow_auto_publish_of_stable_crates,
         ..
     }: Options,
@@ -224,24 +231,31 @@ fn traverse_dependencies_and_find_crates_for_publishing(
         }
         let num_crates_for_publishing_without_dependencies = changed_crate_names_to_publish.len();
         let package = package_by_name(meta, crate_name)?;
-        depth_first_traversal(
+        let skipped = depth_first_traversal(
             meta,
             ctx,
             allow_auto_publish_of_stable_crates,
             &mut seen,
             &mut changed_crate_names_to_publish,
             package,
+            verbose,
         )?;
+        if !verbose && skipped > 0 {
+            log::info!(
+                "Skipped {} dependent crates as they didn't change since their last release. Use --verbose/-v to see much more.",
+                skipped
+            );
+        }
         if num_crates_for_publishing_without_dependencies == changed_crate_names_to_publish.len() {
             let crate_package = package_by_name(meta, crate_name)?;
-            if !git::has_changed_since_last_release(crate_package, ctx)? {
+            if !git::has_changed_since_last_release(crate_package, ctx, verbose)? {
                 log::info!(
                     "Skipping provided {} v{} hasn't changed since last released",
                     crate_package.name,
                     crate_package.version
                 );
+                continue;
             }
-            continue;
         }
         changed_crate_names_to_publish.push(crate_name.to_owned());
         seen.insert(crate_name.to_owned());
@@ -256,22 +270,25 @@ fn depth_first_traversal(
     seen: &mut BTreeSet<String>,
     changed_crate_names_to_publish: &mut Vec<String>,
     package: &Package,
-) -> anyhow::Result<()> {
+    verbose: bool,
+) -> anyhow::Result<usize> {
+    let mut skipped = 0;
     for dependency in package.dependencies.iter().filter(|d| d.kind == DependencyKind::Normal) {
         if seen.contains(&dependency.name) || !is_workspace_member(meta, &dependency.name) {
             continue;
         }
         seen.insert(dependency.name.clone());
         let dep_package = package_by_name(meta, &dependency.name)?;
-        depth_first_traversal(
+        skipped += depth_first_traversal(
             meta,
             ctx,
             allow_auto_publish_of_stable_crates,
             seen,
             changed_crate_names_to_publish,
             dep_package,
+            verbose,
         )?;
-        if git::has_changed_since_last_release(dep_package, ctx)? {
+        if git::has_changed_since_last_release(dep_package, ctx, verbose)? {
             if dep_package.version.major == 0 || allow_auto_publish_of_stable_crates {
                 log::info!(
                     "Adding {} v{} to set of published crates as it changed since last release",
@@ -287,14 +304,17 @@ fn depth_first_traversal(
                 );
             }
         } else {
-            log::info!(
-                "{} v{}  - skipped release as it didn't change",
-                dep_package.name,
-                dep_package.version
-            );
+            if verbose {
+                log::info!(
+                    "{} v{}  - skipped release as it didn't change",
+                    dep_package.name,
+                    dep_package.version
+                );
+            }
+            skipped += 1;
         }
     }
-    Ok(())
+    Ok(skipped)
 }
 
 fn dependency_tree_has_link_to_existing_crate_names(
@@ -396,6 +416,7 @@ fn hops_for_dependency_to_link_back_to_publishee<'a>(
     None
 }
 
+#[allow(clippy::ptr_arg)]
 fn select_publishee_bump_spec<'a>(name: &String, ctx: &'a Context) -> &'a str {
     if ctx.crate_names.contains(name) {
         &ctx.bump
@@ -446,7 +467,7 @@ fn bump_to_valid_version(
         match ctx.index.crate_(&publishee.name) {
             Some(existing_release) => {
                 let existing_version = semver::Version::parse(existing_release.latest_version().version())?;
-                if &existing_version >= &new_version {
+                if existing_version >= new_version {
                     bail!(
                     "Latest published version of '{}' is {}, the new version is {}. Consider using --bump <level> or --bump-dependencies <level>.",
                     publishee.name,
